#! /usr/bin/perl -p0777

## SOLUTION.pl (commented version of piglatin.pl)

## A small perl program by Chris Thorman that converts all English
## words in its inupt stream to Pig Latin.

## Here's how it defines an English word:

## - Must contain only letters [a-zA-Z]; no accented characters.
## - Must not be in ALL UPPER-CASE, as with an Acronym or some HTML tags.
## - Must have some vowels in it.
## - Must start and end at Perl-ish word boundaries (\b -- eliminating
##   words with underscores_separating_them).

## Get a string of all vowels $v...
$v=aeiou,
	
	## ...and a string of non-vowels $n by subtracting the vowels from
	## a..z.  Note only true obfuscation in this code: using b..z
	## instead of a..z. :-)
	
	(($n="@{[b..z]}")=~s/[$v\s]//g);	

## Then use these two strings to form a regular expression that finds
## legal English words, parsing them into the segments that would be
## necessary to pig-latinize the word, and bypassing any words that
## don't meet the minimum basic criteria for pig-latinization.

## This regex would perform the following manipulations:

## The ==> Ethay
## Quick ==> Ickquay
## quick ==> ickquay
## brown ==> ownbray
## Fox ==> Oxfay
## fox ==> oxfay
## over ==> overay
## DOGS ==> DOGS (untouched : presumed acronym)
## by ==> ybay

s/\b				## At the beginning of a word...
	(
	 (((Q)|(q))u) |	## Look for Qu (and set $4) or qu ...
	 ([\U$n\E]) |	## Or an upper-case non-vowel (and set $6)
	 ([$n])			## Or a lower-case non-vowel
	 ) ?			## All of the above are optional....
					## ... so the remainder of the word then presumably contains:
	([$n]*)			## an optional sequence of additional non-vowels (as $8) ...
	(				## ... followed by: (as $9)
	 ([$v\U$v\EyY])	## A vowel or y (of any case since it may be the initial letter of the word)...
	 [a-z]*			## ... possibly followed by any sequence of lower-case letters.
	 )
	\b				## If all of the above matches, and the match ends at a word boundary...
	/				## then replace the word with:
	
    (
	 $4 || $6 ?		## Uppercase Qu or Uppercase non-vowel? 
	 "\u$9" : 		## ... then upper-case the first letter of $9 (middle of word)
	 "$9"			## ... else just start word with its existing case.
	 ) 
	. 				## Then concatenate onto that...
	"\L$1$8\Eay"	## the first part of the word ($1) with opening char
					## plus the optional non-vowel sequence ($8), all lower-cased, 
					## plus the traditional pig-Latin ending "ay".
	/egosx;

